<?php
/**
 * Loulou66
 * LpsSpecials module for Prestashop
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php*
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Loulou66.fr <contact@loulou66.fr>
 *  @copyright loulou66.fr
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;

if (!defined('_PS_VERSION_')) {
    exit;
}

class LpsSpecials extends Module implements WidgetInterface
{
    public $templateFile;

    public function __construct()
    {
        $this->name = 'lpsspecials';
        $this->author = 'Loulou66';
        $this->version = '8.0.2';
        $this->tab = 'front_office_features';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.7.0', 'max' => _PS_VERSION_];
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('LPS Specials Carousel');
        $this->description = $this->l('Add a block displaying your store special offers as a CAROUSEL');
        $this->templateFile = 'module:lpsspecials/views/templates/hook/lpsspecials.tpl';
        $this->module_key = '05d24076bb4af381c7058f03835491eb';
    }
    public function install()
    {
        $this->_clearCache('*');
        return parent::install()
            && Configuration::updateValue('LPSSPECIALS_ENABLE', 1)
            && Configuration::updateValue('LPSSPECIALS_PROD_DISPLAY', 20)
            && Configuration::updateValue('LPSSPECIALS_PROD_PER_ROW', 4)
            && Configuration::updateValue('LPSSPECIALS_LOOP', 1)
            && Configuration::updateValue('LPSSPECIALS_AUTOPLAY', 1)
            && Configuration::updateValue('LPSSPECIALS_RANDOM', 0)
            && $this->registerHook('actionAdminControllerSetMedia')
            && $this->registerHook('actionFrontControllerSetMedia')
            && $this->registerHook('actionProductAdd')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionProductDelete')
            && $this->registerHook('displayHome')
            && $this->registerHook('displayFooterProduct');
    }
    public function uninstall()
    {
        $this->_clearCache('*');
        return parent::uninstall()
            && Configuration::deleteByName('LPSSPECIALS_ENABLE')
            && Configuration::deleteByName('LPSSPECIALS_PROD_DISPLAY')
            && Configuration::deleteByName('LPSSPECIALS_PROD_PER_ROW')
            && Configuration::deleteByName('LPSSPECIALS_LOOP')
            && Configuration::deleteByName('LPSSPECIALS_AUTOPLAY')
            && Configuration::deleteByName('LPSSPECIALS_RANDOM');
    }
    public function isUsingNewTranslationSystem()
    {
        return false;
    }
    public function hookActionProductAdd($params)
    {
        $this->_clearCache('*');
    }
    public function hookActionProductUpdate($params)
    {
        $this->_clearCache('*');
    }
    public function hookActionProductDelete($params)
    {
        $this->_clearCache('*');
    }
    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        return parent::_clearCache($this->templateFile);
    }
    public function hookActionAdminControllerSetMedia($params)
    {
        if ($this->context->controller->controller_name == 'AdminModules'
            && Tools::getValue('configure') == $this->name
        ) {
            $this->context->controller->addCSS($this->_path . 'views/css/' . $this->name . 'admin.css');
            $this->context->controller->addJS($this->_path . 'views/js/' . $this->name . 'admin.js');
        }
    }
    public function getContent()
    {
        $html = '';
        if (Tools::isSubmit('submitlpsspecials')) {
            if (!Validate::isInt((int) Tools::getValue('LPSSPECIALS_PROD_DISPLAY'))
                || (int) Tools::getValue('LPSSPECIALS_PROD_DISPLAY') <= 0
            ) {
                $html .= $this->displayError($this->l('The « Products to display » field is invalid. Please enter a positive number.'));
            } else {
                Configuration::updateValue('LPSSPECIALS_ENABLE', (int) Tools::getValue('LPSSPECIALS_ENABLE'));
                Configuration::updateValue(
                    'LPSSPECIALS_PROD_DISPLAY',
                    (int) Tools::getValue('LPSSPECIALS_PROD_DISPLAY')
                );
                Configuration::updateValue(
                    'LPSSPECIALS_PROD_PER_ROW',
                    (int) Tools::getValue('LPSSPECIALS_PROD_PER_ROW')
                );
                Configuration::updateValue('LPSSPECIALS_LOOP', (int) Tools::getValue('LPSSPECIALS_LOOP'));
                Configuration::updateValue('LPSSPECIALS_AUTOPLAY', (int) Tools::getValue('LPSSPECIALS_AUTOPLAY'));
                Configuration::updateValue('LPSSPECIALS_RANDOM', (int) Tools::getValue('LPSSPECIALS_RANDOM'));
                $this->_clearCache('*');
                $html .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $html . $this->renderForm();
    }
    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable carousel'),
                        'name' => 'LPSSPECIALS_ENABLE',
                        'is_bool' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Products to display'),
                        'name' => 'LPSSPECIALS_PROD_DISPLAY',
                        'desc' => $this->l('Determine the total number of products to display in the carousel'),
                        'hint' => $this->l('Determine the total number of products to display in the carousel'),
                        'form_group_class' => 'LPSSPECIALS_ENABLE',
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Products visible'),
                        'html_content' => $this->displayProdPerRow(),
                        'name' => '',
                        'desc' => $this->l('Define the number of products visible'),
                        'hint' => $this->l('Define the number of products visible'),
                        'form_group_class' => 'LPSSPECIALS_ENABLE',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Loop'),
                        'name' => 'LPSSPECIALS_LOOP',
                        'is_bool' => true,
                        'form_group_class' => 'LPSSPECIALS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Autoplay'),
                        'name' => 'LPSSPECIALS_AUTOPLAY',
                        'is_bool' => true,
                        'form_group_class' => 'LPSSPECIALS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Random display'),
                        'name' => 'LPSSPECIALS_RANDOM',
                        'hint' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'desc' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'is_bool' => true,
                        'form_group_class' => 'LPSSPECIALS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitlpsspecials';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name .
            '&tab_module=' . $this->tab .
            '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        return $helper->generateForm([$fields_form]);
    }
    public function getConfigFieldsValues()
    {
        return [
            'LPSSPECIALS_ENABLE' => (int) Tools::getValue(
                'LPSSPECIALS_ENABLE',
                Configuration::get('LPSSPECIALS_ENABLE'),
            ),
            'LPSSPECIALS_PROD_DISPLAY' => (int) Tools::getValue(
                'LPSSPECIALS_PROD_DISPLAY',
                Configuration::get('LPSSPECIALS_PROD_DISPLAY'),
            ),
            'LPSSPECIALS_LOOP' => (int) Tools::getValue(
                'LPSSPECIALS_LOOP',
                Configuration::get('LPSSPECIALS_LOOP'),
            ),
            'LPSSPECIALS_AUTOPLAY' => (int) Tools::getValue(
                'LPSSPECIALS_AUTOPLAY',
                Configuration::get('LPSSPECIALS_AUTOPLAY'),
            ),
            'LPSSPECIALS_RANDOM' => (int) Tools::getValue(
                'LPSSPECIALS_RANDOM',
                Configuration::get('LPSSPECIALS_RANDOM'),
            ),
        ];
    }
    public function displayProdPerRow()
    {
        $tpl = _PS_MODULE_DIR_ . $this->name . '/views/templates/admin/lpssprodperrow.tpl';
        $lpsbsprodperrow = $this->context->smarty->createTemplate($tpl);
        $lpsbsprodperrow->assign(
            [
                'lpss_prodperrow' => (int) Tools::getValue(
                    'LPSSPECIALS_PROD_PER_ROW',
                    Configuration::get('LPSSPECIALS_PROD_PER_ROW')
                ),
            ]
        );
        return $lpsbsprodperrow->fetch();
    }
    public function hookActionFrontControllerSetMedia()
    {
        if (!$this->active || !Configuration::get('LPSSPECIALS_ENABLE')) {
            return false;
        }
        Media::addJsDef(
            [
                'lpss_productsPerRow' => Configuration::get('LPSSPECIALS_PROD_PER_ROW'),
                'lpss_loop' => Configuration::get('LPSSPECIALS_LOOP') ? true : false,
                'lpss_autoplay' => Configuration::get('LPSSPECIALS_AUTOPLAY') ? true : false,
            ]
        );
        $this->context->controller->registerStylesheet(
            'modules-lpsspecialsfrontcss',
            'modules/' . $this->name . '/views/css/lpsspecialsfront.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselcss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.carousel.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselthemecss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.theme.default.css'
        );
        $this->context->controller->registerJavascript(
            'modules-owlcarouseljs',
            'modules/' . $this->name . '/libs/owlcarousel/js/owl.carousel.min.js'
        );
        $this->context->controller->registerJavascript(
            'modules-lpsspecialsfrontjs',
            'modules/' . $this->name . '/views/js/lpsspecials.js'
        );
    }
    public function renderWidget($hookName, array $configuration)
    {
        if (!$this->active || !Configuration::get('LPSSPECIALS_ENABLE')) {
            return false;
        }
        $lpsspecialsrandom = Configuration::get('LPSSPECIALS_RANDOM');
        if (!$lpsspecialsrandom) {
            if (!$this->isCached($this->templateFile, $this->getCacheId('lpsspecials'))) {
                $variables = $this->getWidgetVariables($hookName, $configuration);
                if (empty($variables)) {
                    return false;
                }
                $this->context->smarty->assign($variables);
            }
            return $this->fetch($this->templateFile, $this->getCacheId('lpsspecials'));
        } else {
            $variables = $this->getWidgetVariables($hookName, $configuration);
            if (empty($variables)) {
                return false;
            }
            $this->context->smarty->assign($variables);
            return $this->fetch($this->templateFile);
        }
    }
    public function getWidgetVariables($hookName, array $configuration)
    {
        $lpsspecials = $this->getSpecials();
        if (count($lpsspecials) <= Configuration::get('LPSSPECIALS_PROD_PER_ROW')) {
            $this->context->smarty->assign(['lpss_hidenav' => true]);
        }
        if (!empty($lpsspecials)) {
            return ['lpsspecials' => $lpsspecials];
        }
        return false;
    }
    protected function getSpecials()
    {
        $lpsspecials = false;
        $nbProducts = Configuration::get('LPSSPECIALS_PROD_DISPLAY');
        $lpsspecials = Product::getPricesDrop(
            (int) Context::getContext()->language->id,
            1,
            (int) $nbProducts,
            false,
            null,
            'ASC'
        );
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $products_for_template = [];
        if (Configuration::get('LPSSPECIALS_RANDOM')) {
            shuffle($lpsspecials);
        }
        if (is_array($lpsspecials)) {
            foreach ($lpsspecials as $rawProduct) {
                $products_for_template[] = $presenter->present(
                    $presentationSettings,
                    $assembler->assembleProduct($rawProduct),
                    $this->context->language
                );
            }
        }
        return $products_for_template;
    }
}
